==> apache-cassandra-3.0.32-src.tar.gz: apache-cassandra-3.0.32-src/src/java/org/apache/cassandra/net/AsyncOneResponse.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cassandra.net;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

/**
 * A callback specialized for returning a value from a single target; that is, this is for messages
 * that we only send to one recipient.
 */
public class AsyncOneResponse<T> implements IAsyncCallback<T>
{
    private T result;
    private boolean done;
    private final long start = System.nanoTime();

    public T get(long timeout, TimeUnit tu) throws TimeoutException
    {
        timeout = tu.toNanos(timeout);
        boolean interrupted = false;
        try
        {
            synchronized (this)
            {
                while (!done)
                {
                    try
                    {
                        long overallTimeout = timeout - (System.nanoTime() - start);
                        if (overallTimeout <= 0)
                        {
                            throw new TimeoutException("Operation timed out.");
                        }
                        TimeUnit.NANOSECONDS.timedWait(this, overallTimeout);
                    }
                    catch (InterruptedException e)
                    {
                        interrupted = true;
                    }
                }
            }
        }
        finally
        {
            if (interrupted)
            {
                Thread.currentThread().interrupt();
            }
        }
        return result;
    }

    public synchronized void response(MessageIn<T> response)
    {
        if (!done)
        {
            result = response.payload;
            done = true;
            this.notifyAll();
        }
    }

    public boolean isLatencyForSnitch()
    {
        return false;
    }
}

==> apache-cassandra-4.0.19-src.tar.gz: apache-cassandra-4.0.19-src/src/java/org/apache/cassandra/net/AsyncOneResponse.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cassandra.net;

import com.google.common.annotations.VisibleForTesting;

import io.netty.util.concurrent.ImmediateEventExecutor;

/**
 * A callback specialized for returning a value from a single target; that is, this is for messages
 * that we only send to one recipient.
 */
public class AsyncOneResponse<T> extends AsyncPromise<T> implements RequestCallback<T>
{
    public AsyncOneResponse()
    {
        super(ImmediateEventExecutor.INSTANCE);
    }

    public void onResponse(Message<T> response)
    {
        setSuccess(response.payload);
    }

    @VisibleForTesting
    public static <T> AsyncOneResponse<T> immediate(T value)
    {
        AsyncOneResponse<T> response = new AsyncOneResponse<>();
        response.setSuccess(value);
        return response;
    }
}
